/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.drawables;

import java.lang.Double;

import net.moioli.chart.DrawableI;
import net.moioli.chart.GraphicsI;
import net.moioli.chart.Scale;

/**
 * A general implementation of the DrawableI and PlaneCurveI interfaces.
 * Includes empirical minY(), maxY(), minX() and maxX() methods.
 * 
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public abstract class AbstractPlaneCurve implements PlaneCurveI, DrawableI {
    /** The maximum value the parameter t is given */
    private double maxT = 2 * Math.PI;

    /**
     * Parameter for the emprical min/max detection. The function will be
     * re-evaluated every (getMaxT())/ACCURACY wide step.
     */
    private static final double ACCURACY = 50;

    /**
     * Default constructor, requires the maximum parameter value.
     * 
     * @param maxT
     *            the maximum parameter value
     */
    public AbstractPlaneCurve(double maxT) {
        super();
        this.maxT = maxT;
    }

    /**
     * @see net.moioli.drawables.PlaneCurveI#getMaxT()
     */
    public double getMaxT() {
        return this.maxT;
    }

    /**
     * @see net.moioli.drawables.PlaneCurveI#setMaxT(double t)
     */
    public void setMaxT(double t) {
        this.maxT = t;
    }

    /**
     * @see net.moioli.drawables.PlaneCurveI#x(double t)
     */
    public abstract double x(double t);

    /**
     * @see net.moioli.drawables.PlaneCurveI#y(double t)
     */
    public abstract double y(double t);

    /**
     * Returns this.x(x), or NaN if an exception occured while computing.
     * 
     * @param t
     *            the parameter
     * @return x(t) or NaN
     */
    private double safeX(double t) {
        try {
            return this.x(t);
        } catch (ArithmeticException e) {
            return Double.NaN;
        }
    }

    /**
     * Returns this.y(x), or NaN if an exception occured while computing.
     * 
     * @param t
     *            the parameter
     * @return y(t) or NaN
     */
    private double safeY(double t) {
        double result;
        try {
            result = this.y(t);
        } catch (ArithmeticException e) {
            result = Double.NaN;
        }
        return result;
    }

    /**
     * Returns the parameter value in which the function has an x minimum with
     * an empirical algorithm. While this method is accurate enough in most
     * cases, it should be overridden whenever possible.
     * 
     * @return the parameter value in which the function has an x minimum
     */
    public double getXMinimum() {
        double step = (getMaxT()) / ACCURACY;
        double acc = 0;
        double minValue = Double.POSITIVE_INFINITY;
        double result = 0;
        double value;
        for (int i = 0; i < ACCURACY; i++) {
            value = this.safeX(acc);
            if (value < minValue) {
                minValue = value;
                result = acc;
            }
            acc += step;
        }
        return result;
    }

    /**
     * Returns the parameter value in which the function has an x maximum with
     * an empirical algorithm. While this method is accurate enough in most
     * cases, it should be overridden whenever possible.
     * 
     * @return the parameter value in which the function has an x maximum
     */
    public double getXMaximum() {
        double step = (getMaxT()) / ACCURACY;
        double acc = 0;
        double maxValue = Double.NEGATIVE_INFINITY;
        double result = 0;
        double value;
        for (int i = 0; i < ACCURACY; i++) {
            value = this.safeX(acc);
            if (value > maxValue) {
                maxValue = value;
                result = acc;
            }
            acc += step;
        }
        return result;
    }

    /**
     * Returns the parameter value in which the function has an y minimum with
     * an empirical algorithm. While this method is accurate enough in most
     * cases, it should be overridden whenever possible.
     * 
     * @return the parameter value in which the function has an y minimum
     */
    public double getYMinimum() {
        double step = (getMaxT()) / ACCURACY;
        double acc = 0;
        double minValue = Double.POSITIVE_INFINITY;
        double result = 0;
        double value;
        for (int i = 0; i < ACCURACY; i++) {
            value = this.safeY(acc);
            if (value < minValue) {
                minValue = value;
                result = acc;
            }
            acc += step;
        }
        return result;
    }

    /**
     * Returns the parameter value in which the function has an y maximum with
     * an empirical algorithm. While this method is accurate enough in most
     * cases, it should be overridden whenever possible.
     * 
     * @return the parameter value in which the function has an y maximum
     */
    public double getYMaximum() {
        double step = (getMaxT()) / ACCURACY;
        double acc = 0;
        double maxValue = Double.NEGATIVE_INFINITY;
        double result = 0;
        double value;
        for (int i = 0; i < ACCURACY; i++) {
            value = this.safeY(acc);
            if (value > maxValue) {
                maxValue = value;
                result = acc;
            }
            acc += step;
        }
        return result;
    }

    /**
     * Returns the minimum x value that the function takes.
     * 
     * @return the minimum x
     */
    public double minX() {
        return this.safeY(this.getYMinimum());
    }

    /**
     * Returns the minimum y value that the function takes.
     * 
     * @return the minimum y
     */
    public double maxX() {
        return this.safeY(this.getYMaximum());
    }

    /**
     * @see net.moioli.chart.DrawableI#minY(double, double)
     */
    public double minY(double minX, double maxX) {
        return this.safeY(this.getYMinimum());
    }

    /**
     * @see net.moioli.chart.DrawableI#maxY(double, double)
     */
    public double maxY(double minX, double maxX) {
        return this.safeY(this.getYMaximum());
    }

    /**
     * @see net.moioli.chart.DrawableI#drawMe(GraphicsI g, Scale s, Object c)
     */
    public void drawMe(GraphicsI g, Scale s, Object c) {
        double minX = s.getMinX();
        double maxX = s.getMaxX();
        double minY = s.getMinY();
        double maxY = s.getMaxY();
        double thisMaxT = this.getMaxT();
        double step = s.getStep();
        double j;
        g.setColor(c);

        double thisX = minX;
        double previousX = minX;
        double thisY = 0;
        double previousY = 0;

        for (j = 0; j <= thisMaxT; j += step) {
            previousX = safeX(j);
            previousY = safeY(j);
            if ((!Double.isNaN(previousY)) && ((!Double.isNaN(previousX)))) {
                break;
            }
        }

        // j is already inited
        for (; j <= thisMaxT; j += step) {
            thisX = safeX(j);
            thisY = safeY(j);
            if ((!Double.isNaN(thisX)) && ((!Double.isNaN(thisY)))) {
                if (thisX > minX && thisX < maxX && thisY > minY
                        && thisY < maxY) {
                    g.drawLine(s.getX(previousX), s.getY(previousY), s
                            .getX(thisX), s.getY(thisY));
                }
                previousX = thisX;
                previousY = thisY;
            }
        }
    }
}
