/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.drawables;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.lang.Number;
import java.lang.Double;

import net.moioli.chart.DrawableI;
import net.moioli.chart.GraphicsI;
import net.moioli.chart.Scale;

/**
 * A nice class to draw a pie out of a Map table of data. <br />
 * It will draw a pie with the values' percentages and will use the Map keys as
 * lables. <br />
 * Attention: this class expects you use Number derived objects as the map
 * values (like Double or Integer). Not doing so might raise
 * ClassCastExceptions.
 * 
 * @see java.lang.Number
 * @see java.util.Map
 * 
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public class Pie extends TreeMap implements DrawableI {
    /** Number of vertices of each "pie slice" */
    private final static int VERTICES = 30;

    /**
     * Default constructor. 
     */
    public Pie() {
        super();
    }

    /**
     * @see net.moioli.chart.DrawableI#drawMe(net.moioli.chart.GraphicsI,
     *      net.moioli.chart.Scale, java.lang.Object)
     */
    public void drawMe(GraphicsI g, Scale s, Object c) {
        double total = 0;
        Collection es = super.entrySet();
        Iterator i = es.iterator();
        while (i.hasNext()) {
            total += ((Number) ((Map.Entry) i.next()).getValue()).doubleValue();
        }
        i = es.iterator();
        double beginAngle = 0;
        while (i.hasNext()) {
            double currentValue = ((Number) ((Map.Entry) i.next()).getValue())
                    .doubleValue();
            double currentAngle = currentValue / total * 2 * Math.PI;
            int[] x = new int[VERTICES + 2];
            int[] y = new int[VERTICES + 2];

            for (int j = 0; j <= VERTICES; j++) {
                double littleAngle = beginAngle + currentAngle * j / VERTICES;
                x[j] = (int) Math.round(s.getScreenMaxX()
                        * Math.cos(littleAngle) / 2 + s.getScreenMaxX() / 2);
                y[j] = (int) Math.round(s.getScreenMaxY()
                        * Math.sin(littleAngle) / 2 + s.getScreenMaxY() / 2);
            }

            x[VERTICES + 1] = s.getX(0);
            y[VERTICES + 1] = s.getY(0);

            // Pick a random color and draw
            g.setColor(g.getRandomColor());
            g.fillPolygon(x, y, VERTICES + 2);

            beginAngle += currentAngle;
        }

        i = es.iterator();
        beginAngle = 0;
        g.setColor(g.getBlackColor());
        while (i.hasNext()) {
            Map.Entry currentEntry = (Map.Entry) i.next();
            Object currentKey = currentEntry.getKey();
            double currentValue = ((Number) currentEntry.getValue())
                    .doubleValue();
            double currentAngle = currentValue / total * 2 * Math.PI;
            double mediumAngle = beginAngle + currentAngle / 2;
            String percent = Double.toString(currentValue / total * 100) + "%";
            if (percent.length() > 6) {
                percent = percent.substring(0, 6) + "%";
            }
            String label = currentKey + ": " + percent;
            int stringX = (int) Math.round(s.getScreenMaxX()
                    * Math.cos(mediumAngle) / 2 / 2 + s.getScreenMaxX() / 2);
            int stringY = (int) Math.round(s.getScreenMaxY()
                    * Math.sin(mediumAngle) / 2 / 2 + s.getScreenMaxY() / 2);
            g.drawString(label, stringX, stringY);
            beginAngle += currentAngle;
        }
    }

    /**
     * @see net.moioli.chart.DrawableI#minY(double, double)
     */
    public double minY(double minX, double maxX) {
        return Double.NaN;
    }

    /**
     * @see net.moioli.chart.DrawableI#maxY(double, double)
     */
    public double maxY(double minX, double maxX) {
        return Double.NaN;
    }
}
